@@ -221,6 +221,15 @@ class GPGTestCase(unittest.TestCase):
         private_keys = self.gpg.list_keys(secret=True)
         self.assertTrue(is_list_with_len(private_keys, 1),
                         "1-element list expected")
+        # Now do the same test, but using keyring and secret_keyring arguments
+        hd = os.path.join(os.getcwd(), 'keys')
+        gpg = gnupg.GPG(gnupghome=hd, gpgbinary=GPGBINARY,
+                        keyring=os.path.join(hd, 'pubring.gpg'),
+                        secret_keyring=os.path.join(hd, 'secring.gpg'))
+        public_keys_2 = gpg.list_keys()
+        self.assertEqual(public_keys_2, public_keys)
+        private_keys_2 = gpg.list_keys(secret=True)
+        self.assertEqual(private_keys_2, private_keys)
 
     def test_encryption_and_decryption(self):
         "Test that encryption and decryption works"
@@ -322,6 +331,7 @@ class GPGTestCase(unittest.TestCase):
         self.assertFalse(sig, "Bad passphrase should fail")
         sig = self.gpg.sign(data, keyid=key.fingerprint, passphrase='aable')
         self.assertTrue(sig, "Good passphrase should succeed")
+        self.assertTrue(sig.hash_algo)
         verified = self.gpg.verify(sig.data)
         if key.fingerprint != verified.fingerprint:
             logger.debug("key: %r", key.fingerprint)
@@ -339,6 +349,7 @@ class GPGTestCase(unittest.TestCase):
                                  passphrase='aable')
         data_file.close()
         self.assertTrue(sig, "File signing should succeed")
+        self.assertTrue(sig.hash_algo)
         try:
             file = gnupg._make_binary_stream(sig.data, self.gpg.encoding)
             verified = self.gpg.verify_file(file)
@@ -355,6 +366,7 @@ class GPGTestCase(unittest.TestCase):
                                  passphrase='aable', detach=True)
         data_file.close()
         self.assertTrue(sig, "File signing should succeed")
+        self.assertTrue(sig.hash_algo)
         try:
             file = gnupg._make_binary_stream(sig.data, self.gpg.encoding)
             verified = self.gpg.verify_file(file, 'random_binary_data')
